package ru.ldeloff.servermonitorbot.service.command;

import lombok.Data;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import org.telegram.telegrambots.meta.api.objects.Update;
import org.telegram.telegrambots.meta.exceptions.TelegramApiException;
import ru.ldeloff.servermonitorbot.controller.TelegramBotController;
import ru.ldeloff.servermonitorbot.model.User;
import ru.ldeloff.servermonitorbot.service.role.RoleService;
import ru.ldeloff.servermonitorbot.service.user.UserService;

@Data
@Slf4j
@RequiredArgsConstructor
public abstract class CommandTemplate {
    final UserService userService;
    final RoleService roleService;
    long expectedRole = 2L;
    public abstract SendMessage actionForAuth(User user, SendMessage message);

    public SendMessage actionForNotAuth(User user, SendMessage message) {
        message.setText("У пользователя " + user.getLogin() + " (" + user.getTelegramId()
                + ") недостаточно прав для выполнения этой команды");
        return message;
    }
    public void execute(Update update, TelegramBotController bot) {
        User user = getUser(update);
        String message = getMessage(update);
        SendMessage answer = new SendMessage();
        answer.setChatId(user.getTelegramId());
        answer.setText(message);
        if (user.getRole().getId() <= expectedRole) {
            logSuccess(user, message);
            sendMessage(actionForAuth(user, answer), bot);
        } else {
            logNotAuth(user, message);
            sendMessage(actionForNotAuth(user, answer), bot);
        }
    }
    protected void logSuccess(User user, String message) {
        log.info("Получена команда '" + message + "' от " + user.getLogin()
                + " (" + user.getTelegramId() + "). OK");
    }
    protected void logNotAuth(User user, String message) {
        log.warn("Получена команда '" + message + "' от " + user.getLogin()
                + " (" + user.getTelegramId() + "). Нет прав");
    }
    private User getUser(Update update) {
        long id = -1L;
        String login = null;
        if (update.hasMessage()) {
            id = update.getMessage().getChat().getId();
            login = update.getMessage().getChat().getUserName();
        } else if (update.hasCallbackQuery()) {
            id = update.getCallbackQuery().getMessage().getChat().getId();
            login = update.getCallbackQuery().getMessage().getChat().getUserName();
        }
        User user = userService.getByTelegramId(id);
        if (user == null) {
            user = new User();
            user.setTelegramId(id);
            user.setRole(roleService.findRoleById(3L));
        }
        user.setLogin(login);
        return user;
    }

    private String getMessage(Update update) {
        if (update.hasMessage()) {
            return update.getMessage().getText();
        } else if (update.hasCallbackQuery()) {
            return update.getCallbackQuery().getData();
        }
        return "";
    }

    private void sendMessage(SendMessage message, TelegramBotController bot) {
        try {
            bot.execute(message);
        } catch (TelegramApiException e) {
            e.printStackTrace();
            log.warn(e.getMessage());
        }
    }
}
